package cert;

import java.io.*;
import java.security.*;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;

import java.math.BigInteger;
import java.util.Date;

import javax.security.auth.x500.X500Principal;

import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.openssl.PEMWriter;
import org.bouncycastle.x509.X509V3CertificateGenerator;

public class CreateSelfCertificate {

	public static X509Certificate generateV3CertificateBC(KeyPair pair) 
	throws InvalidKeyException, NoSuchProviderException, SignatureException, 
		NoSuchAlgorithmException, CertificateEncodingException, IllegalStateException {
		
		X509V3CertificateGenerator certGen = new X509V3CertificateGenerator();
		
		certGen.setSerialNumber(BigInteger.valueOf(System.currentTimeMillis()));
		certGen.setIssuerDN(new X500Principal("CN=Test Certificate"));
		certGen.setNotBefore(new Date(System.currentTimeMillis() - 50000));
		certGen.setNotAfter(new Date(System.currentTimeMillis() + 50000));
		certGen.setSubjectDN(new X500Principal("CN=Test Certificate"));
		certGen.setPublicKey(pair.getPublic());
		certGen.setSignatureAlgorithm("SHA1WithRSAEncryption");
		
		certGen.addExtension(X509Extensions.SubjectAlternativeName, false, 
					new GeneralNames(new GeneralName(GeneralName.rfc822Name, "test@test.test")));

		return certGen.generate(pair.getPrivate(), "BC");
	}
	
	
	public static void exportCertificate(X509Certificate cert, String filename)
	throws CertificateEncodingException, IOException{
		PEMWriter pemWr = new PEMWriter(new OutputStreamWriter(new FileOutputStream(filename)));
		pemWr.writeObject(cert);
		pemWr.close();	
	}
	
	public static void main(String[] args){
		
		// modificare la stringa path
		String path = ""; 
		String filename = path + "myCert.crt";

		try {
			// Building keys
			System.out.println("Building keys...");
			KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance("RSA", "BC");
			keyPairGen.initialize(1024);
			KeyPair keys = keyPairGen.generateKeyPair();
			
			// Building my SelfCertificate
			System.out.println("Building certificate...");
			X509Certificate cert = generateV3CertificateBC(keys);
			
			// Basic validation
			System.out.println("Validating dates...");
			cert.checkValidity(new Date());
			System.out.println("Verifying signature...");
			cert.verify(cert.getPublicKey());
			System.out.println("Dates and signature verified.");
			
			System.out.println("Exporting certificate...");
			exportCertificate(cert, filename);
			System.out.println("Certificate exported.");
		}
		
		catch(Exception e){
			e.printStackTrace();
		}	
		
	}
	
	
	
}
