package cert;

import java.io.*;
import java.security.*;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import java.math.BigInteger;
import java.util.Date;

import javax.crypto.Cipher;
import javax.security.auth.x500.X500Principal;

import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.x509.X509V3CertificateGenerator;

import sun.misc.BASE64Encoder;

public class UseCertificate {
	
	public static void main(String[] args){
		
		// modificare la stringa path
		String path = ""; 
		String filename = path + "myCert2.crt";

		try {
			
			// Actor 1
			
			System.out.println("# Actor 1");
			// Building keys
			System.out.println("Building keys...");
			KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance("RSA", "BC");
			keyPairGen.initialize(1024);
			KeyPair keys = keyPairGen.generateKeyPair();
			
			// Building my Certificate
			System.out.println("");
			System.out.println("Building certificate...");
			X509Certificate cert = CreateSelfCertificate.generateV3CertificateBC(keys);
			System.out.println("Certificate built.");
			
			// Exporting my Certificate
			System.out.println("");
			System.out.println("Exporting certificate...");
			CreateSelfCertificate.exportCertificate(cert, filename);
			System.out.println("Certificate exported.");
			
			/**********************************/
			System.out.println("--------");
			// Actor 2
			
			System.out.println("# Actor 2");
			System.out.println("Reading certificate...");
			// Retrieve certificate
			InputStream is = new FileInputStream(filename);
			CertificateFactory fact = CertificateFactory.getInstance("X.509", "BC");			
			X509Certificate retrievedCert = (X509Certificate)fact.generateCertificate(is);
			is.close();
			
			// Retrieve Actor 1's Public Key from certificate
			System.out.println("Retrieving PublicKey...");
			PublicKey pKey = retrievedCert.getPublicKey();
			
			
			System.out.println("----------");
			
			// Trusted communication between the parties
			
			// Actor 2 encrypts a message using Actor 1's public key
			System.out.println("");
			System.out.println("Trusted communication");
			System.out.println("********");
			System.out.println("Actor 2 - encrypting message...");
			String msg = "ciao ciao";
			System.out.println("Actor 2 - msg = " + msg);
			byte[] msgBytes = msg.getBytes();
			Cipher cipherE = Cipher.getInstance("RSA/None/PKCS1Padding", "BC");
			cipherE.init(Cipher.ENCRYPT_MODE, pKey);
			byte[] encryptedMsg = cipherE.doFinal(msgBytes);
			System.out.println("Actor 2 - encrypted msg = " + new BASE64Encoder().encode(encryptedMsg));
			System.out.println("Actor 2 - encrypted message sent.");
			System.out.println("********");
			
			// Actor 1 decrypts received message with his private key
			System.out.println("Actor 1 - decrypting message...");
			Cipher cipherD = Cipher.getInstance("RSA/None/PKCS1Padding", "BC");
			cipherD.init(Cipher.DECRYPT_MODE, keys.getPrivate());
			byte[] decryptedMsg = cipherD.doFinal(encryptedMsg);
			System.out.println("Actor 1 - decrypted msg = " + new String(decryptedMsg));	
			System.out.println("********");
		}
		
		catch(Exception e){
			e.printStackTrace();
		}	
		
	}
	
	
	
}
